@@ -4,11 +4,13 @@ General-purpose fixtures for vdirsyncer's testsuite.
 import logging
 import os
 
+import aiohttp
 import click_log
 import pytest
+import pytest_asyncio
 from hypothesis import HealthCheck
-from hypothesis import settings
 from hypothesis import Verbosity
+from hypothesis import settings
 
 
 @pytest.fixture(autouse=True)
@@ -24,7 +26,6 @@ except ImportError:
     def benchmark():
         return lambda x: x()
 
-
 else:
     del pytest_benchmark
 
@@ -52,3 +53,15 @@ elif os.environ.get("CI", "false").lower() == "true":
     settings.load_profile("ci")
 else:
     settings.load_profile("dev")
+
+
+@pytest_asyncio.fixture
+async def aio_session(event_loop):
+    async with aiohttp.ClientSession() as session:
+        yield session
+
+
+@pytest_asyncio.fixture
+async def aio_connector(event_loop):
+    async with aiohttp.TCPConnector(limit_per_host=16) as conn:
+        yield conn
