@@ -11,7 +11,7 @@ try:
         "url": "https://brutus.lostpackets.de/davical-test/caldav.php/",
     }
 except KeyError as e:
-    pytestmark = pytest.mark.skip("Missing envkey: {}".format(str(e)))
+    pytestmark = pytest.mark.skip(f"Missing envkey: {str(e)}")
 
 
 @pytest.mark.flaky(reruns=5)
@@ -27,7 +27,7 @@ class ServerMixin:
 
     @pytest.fixture
     def get_storage_args(self, davical_args, request):
-        def inner(collection="test"):
+        async def inner(collection="test"):
             if collection is None:
                 return davical_args
 
@@ -39,7 +39,8 @@ class ServerMixin:
                 )
                 s = self.storage_class(**args)
                 if not list(s.list()):
-                    request.addfinalizer(lambda: s.session.request("DELETE", ""))
+                    # See: https://stackoverflow.com/a/33984811
+                    request.addfinalizer(lambda x=s: x.session.request("DELETE", ""))
                     return args
 
             raise RuntimeError("Failed to find free collection.")
