@@ -50,41 +50,6 @@ def test_sync_inexistant_pair(tmpdir, runner):
     assert "pair foo does not exist." in result.output.lower()
 
 
-def test_debug_connections(tmpdir, runner):
-    runner.write_with_general(
-        dedent(
-            """
-    [pair my_pair]
-    a = "my_a"
-    b = "my_b"
-    collections = null
-
-    [storage my_a]
-    type = "filesystem"
-    path = "{0}/path_a/"
-    fileext = ".txt"
-
-    [storage my_b]
-    type = "filesystem"
-    path = "{0}/path_b/"
-    fileext = ".txt"
-    """
-        ).format(str(tmpdir))
-    )
-
-    tmpdir.mkdir("path_a")
-    tmpdir.mkdir("path_b")
-
-    result = runner.invoke(["discover"])
-    assert not result.exception
-
-    result = runner.invoke(["-vdebug", "sync", "--max-workers=3"])
-    assert "using 3 maximal workers" in result.output.lower()
-
-    result = runner.invoke(["-vdebug", "sync"])
-    assert "using 1 maximal workers" in result.output.lower()
-
-
 def test_empty_storage(tmpdir, runner):
     runner.write_with_general(
         dedent(
@@ -388,16 +353,13 @@ def test_ident_conflict(tmpdir, runner):
         'error: foobar: Storage "foo" contains multiple items with the '
         "same UID or even content"
     ) in result.output
-    assert (
-        sorted(
-            [
-                "one.txt" in result.output,
-                "two.txt" in result.output,
-                "three.txt" in result.output,
-            ]
-        )
-        == [False, True, True]
-    )
+    assert sorted(
+        [
+            "one.txt" in result.output,
+            "two.txt" in result.output,
+            "three.txt" in result.output,
+        ]
+    ) == [False, True, True]
 
 
 @pytest.mark.parametrize(
@@ -444,8 +406,8 @@ def test_no_configured_pairs(tmpdir, runner, cmd):
     runner.write_with_general("")
 
     result = runner.invoke([cmd])
-    assert result.output == "critical: Nothing to do.\n"
-    assert result.exception.code == 5
+    assert result.output == ""
+    assert not result.exception
 
 
 @pytest.mark.parametrize(
