@@ -1,16 +1,18 @@
+import asyncio
 from copy import deepcopy
 
+import aiostream
 import hypothesis.strategies as st
 import pytest
 from hypothesis import assume
 from hypothesis.stateful import Bundle
-from hypothesis.stateful import rule
 from hypothesis.stateful import RuleBasedStateMachine
+from hypothesis.stateful import rule
 
 from tests import blow_up
 from tests import uid_strategy
-from vdirsyncer.storage.memory import _random_string
 from vdirsyncer.storage.memory import MemoryStorage
+from vdirsyncer.storage.memory import _random_string
 from vdirsyncer.sync import sync as _sync
 from vdirsyncer.sync.exceptions import BothReadOnly
 from vdirsyncer.sync.exceptions import IdentConflict
@@ -21,10 +23,10 @@ from vdirsyncer.sync.status import SqliteStatus
 from vdirsyncer.vobject import Item
 
 
-def sync(a, b, status, *args, **kwargs):
+async def sync(a, b, status, *args, **kwargs):
     new_status = SqliteStatus(":memory:")
     new_status.load_legacy_status(status)
-    rv = _sync(a, b, new_status, *args, **kwargs)
+    rv = await _sync(a, b, new_status, *args, **kwargs)
     status.clear()
     status.update(new_status.to_legacy_status())
     return rv
@@ -38,45 +40,49 @@ def items(s):
     return {x[1].raw for x in s.items.values()}
 
 
-def test_irrelevant_status():
+@pytest.mark.asyncio
+async def test_irrelevant_status():
     a = MemoryStorage()
     b = MemoryStorage()
     status = {"1": ("1", 1234, "1.ics", 2345)}
-    sync(a, b, status)
+    await sync(a, b, status)
     assert not status
     assert not items(a)
     assert not items(b)
 
 
-def test_missing_status():
+@pytest.mark.asyncio
+async def test_missing_status():
     a = MemoryStorage()
     b = MemoryStorage()
     status = {}
     item = Item("asdf")
-    a.upload(item)
-    b.upload(item)
-    sync(a, b, status)
+    await a.upload(item)
+    await b.upload(item)
+    await sync(a, b, status)
     assert len(status) == 1
     assert items(a) == items(b) == {item.raw}
 
 
-def test_missing_status_and_different_items():
+@pytest.mark.asyncio
+async def test_missing_status_and_different_items():
     a = MemoryStorage()
     b = MemoryStorage()
 
     status = {}
     item1 = Item("UID:1\nhaha")
     item2 = Item("UID:1\nhoho")
-    a.upload(item1)
-    b.upload(item2)
+    await a.upload(item1)
+    await b.upload(item2)
     with pytest.raises(SyncConflict):
-        sync(a, b, status)
+        await sync(a, b, status)
     assert not status
-    sync(a, b, status, conflict_resolution="a wins")
+    await sync(a, b, status, conflict_resolution="a wins")
     assert items(a) == items(b) == {item1.raw}
 
 
-def test_read_only_and_prefetch():
+@pytest.mark.asyncio
+async def test_read_only_and_prefetch():
     a = MemoryStorage()
     b = MemoryStorage()
     b.read_only = True
@@ -84,147 +90,154 @@ def test_read_only_and_prefetch():
     status = {}
     item1 = Item("UID:1\nhaha")
     item2 = Item("UID:2\nhoho")
-    a.upload(item1)
-    a.upload(item2)
+    await a.upload(item1)
+    await a.upload(item2)
 
-    sync(a, b, status, force_delete=True)
-    sync(a, b, status, force_delete=True)
+    await sync(a, b, status, force_delete=True)
+    await sync(a, b, status, force_delete=True)
 
     assert not items(a) and not items(b)
 
 
-def test_partial_sync_error():
+@pytest.mark.asyncio
+async def test_partial_sync_error():
     a = MemoryStorage()
     b = MemoryStorage()
     status = {}
 
-    a.upload(Item("UID:0"))
+    await a.upload(Item("UID:0"))
     b.read_only = True
 
     with pytest.raises(PartialSync):
-        sync(a, b, status, partial_sync="error")
+        await sync(a, b, status, partial_sync="error")
 
 
-def test_partial_sync_ignore():
+@pytest.mark.asyncio
+async def test_partial_sync_ignore():
     a = MemoryStorage()
     b = MemoryStorage()
     status = {}
 
     item0 = Item("UID:0\nhehe")
-    a.upload(item0)
-    b.upload(item0)
+    await a.upload(item0)
+    await b.upload(item0)
 
     b.read_only = True
 
     item1 = Item("UID:1\nhaha")
-    a.upload(item1)
+    await a.upload(item1)
 
-    sync(a, b, status, partial_sync="ignore")
-    sync(a, b, status, partial_sync="ignore")
+    await sync(a, b, status, partial_sync="ignore")
+    await sync(a, b, status, partial_sync="ignore")
 
     assert items(a) == {item0.raw, item1.raw}
     assert items(b) == {item0.raw}
 
 
-def test_partial_sync_ignore2():
+@pytest.mark.asyncio
+async def test_partial_sync_ignore2():
     a = MemoryStorage()
     b = MemoryStorage()
     status = {}
 
-    href, etag = a.upload(Item("UID:0"))
+    href, etag = await a.upload(Item("UID:0"))
     a.read_only = True
 
-    sync(a, b, status, partial_sync="ignore", force_delete=True)
+    await sync(a, b, status, partial_sync="ignore", force_delete=True)
     assert items(b) == items(a) == {"UID:0"}
 
     b.items.clear()
-    sync(a, b, status, partial_sync="ignore", force_delete=True)
-    sync(a, b, status, partial_sync="ignore", force_delete=True)
+    await sync(a, b, status, partial_sync="ignore", force_delete=True)
+    await sync(a, b, status, partial_sync="ignore", force_delete=True)
     assert items(a) == {"UID:0"}
     assert not b.items
 
     a.read_only = False
-    a.update(href, Item("UID:0\nupdated"), etag)
+    await a.update(href, Item("UID:0\nupdated"), etag)
     a.read_only = True
-    sync(a, b, status, partial_sync="ignore", force_delete=True)
+    await sync(a, b, status, partial_sync="ignore", force_delete=True)
     assert items(b) == items(a) == {"UID:0\nupdated"}
 
 
-def test_upload_and_update():
+@pytest.mark.asyncio
+async def test_upload_and_update():
     a = MemoryStorage(fileext=".a")
     b = MemoryStorage(fileext=".b")
     status = {}
 
     item = Item("UID:1")  # new item 1 in a
-    a.upload(item)
-    sync(a, b, status)
+    await a.upload(item)
+    await sync(a, b, status)
     assert items(b) == items(a) == {item.raw}
 
     item = Item("UID:1\nASDF:YES")  # update of item 1 in b
-    b.update("1.b", item, b.get("1.b")[1])
-    sync(a, b, status)
+    await b.update("1.b", item, (await b.get("1.b"))[1])
+    await sync(a, b, status)
     assert items(b) == items(a) == {item.raw}
 
     item2 = Item("UID:2")  # new item 2 in b
-    b.upload(item2)
-    sync(a, b, status)
+    await b.upload(item2)
+    await sync(a, b, status)
     assert items(b) == items(a) == {item.raw, item2.raw}
 
     item2 = Item("UID:2\nASDF:YES")  # update of item 2 in a
-    a.update("2.a", item2, a.get("2.a")[1])
-    sync(a, b, status)
+    await a.update("2.a", item2, (await a.get("2.a"))[1])
+    await sync(a, b, status)
     assert items(b) == items(a) == {item.raw, item2.raw}
 
 
-def test_deletion():
+@pytest.mark.asyncio
+async def test_deletion():
     a = MemoryStorage(fileext=".a")
     b = MemoryStorage(fileext=".b")
     status = {}
 
     item = Item("UID:1")
-    a.upload(item)
+    await a.upload(item)
     item2 = Item("UID:2")
-    a.upload(item2)
-    sync(a, b, status)
-    b.delete("1.b", b.get("1.b")[1])
-    sync(a, b, status)
+    await a.upload(item2)
+    await sync(a, b, status)
+    await b.delete("1.b", (await b.get("1.b"))[1])
+    await sync(a, b, status)
     assert items(a) == items(b) == {item2.raw}
 
-    a.upload(item)
-    sync(a, b, status)
+    await a.upload(item)
+    await sync(a, b, status)
     assert items(a) == items(b) == {item.raw, item2.raw}
-    a.delete("1.a", a.get("1.a")[1])
-    sync(a, b, status)
+    await a.delete("1.a", (await a.get("1.a"))[1])
+    await sync(a, b, status)
     assert items(a) == items(b) == {item2.raw}
 
 
-def test_insert_hash():
+@pytest.mark.asyncio
+async def test_insert_hash():
     a = MemoryStorage()
     b = MemoryStorage()
     status = {}
 
     item = Item("UID:1")
-    href, etag = a.upload(item)
-    sync(a, b, status)
+    href, etag = await a.upload(item)
+    await sync(a, b, status)
 
     for d in status["1"]:
         del d["hash"]
 
-    a.update(href, Item("UID:1\nHAHA:YES"), etag)
-    sync(a, b, status)
+    await a.update(href, Item("UID:1\nHAHA:YES"), etag)
+    await sync(a, b, status)
     assert "hash" in status["1"][0] and "hash" in status["1"][1]
 
 
-def test_already_synced():
+@pytest.mark.asyncio
+async def test_already_synced():
     a = MemoryStorage(fileext=".a")
     b = MemoryStorage(fileext=".b")
     item = Item("UID:1")
-    a.upload(item)
-    b.upload(item)
+    await a.upload(item)
+    await b.upload(item)
     status = {
         "1": (
-            {"href": "1.a", "hash": item.hash, "etag": a.get("1.a")[1]},
-            {"href": "1.b", "hash": item.hash, "etag": b.get("1.b")[1]},
+            {"href": "1.a", "hash": item.hash, "etag": (await a.get("1.a"))[1]},
+            {"href": "1.b", "hash": item.hash, "etag": (await b.get("1.b"))[1]},
         )
     }
     old_status = deepcopy(status)
@@ -233,69 +246,73 @@ def test_already_synced():
     )
 
     for _ in (1, 2):
-        sync(a, b, status)
+        await sync(a, b, status)
         assert status == old_status
         assert items(a) == items(b) == {item.raw}
 
 
 @pytest.mark.parametrize("winning_storage", "ab")
-def test_conflict_resolution_both_etags_new(winning_storage):
+@pytest.mark.asyncio
+async def test_conflict_resolution_both_etags_new(winning_storage):
     a = MemoryStorage()
     b = MemoryStorage()
     item = Item("UID:1")
-    href_a, etag_a = a.upload(item)
-    href_b, etag_b = b.upload(item)
+    href_a, etag_a = await a.upload(item)
+    href_b, etag_b = await b.upload(item)
     status = {}
-    sync(a, b, status)
+    await sync(a, b, status)
     assert status
     item_a = Item("UID:1\nitem a")
     item_b = Item("UID:1\nitem b")
-    a.update(href_a, item_a, etag_a)
-    b.update(href_b, item_b, etag_b)
+    await a.update(href_a, item_a, etag_a)
+    await b.update(href_b, item_b, etag_b)
     with pytest.raises(SyncConflict):
-        sync(a, b, status)
-    sync(a, b, status, conflict_resolution=f"{winning_storage} wins")
+        await sync(a, b, status)
+    await sync(a, b, status, conflict_resolution=f"{winning_storage} wins")
     assert (
         items(a) == items(b) == {item_a.raw if winning_storage == "a" else item_b.raw}
     )
 
 
-def test_updated_and_deleted():
+@pytest.mark.asyncio
+async def test_updated_and_deleted():
     a = MemoryStorage()
     b = MemoryStorage()
-    href_a, etag_a = a.upload(Item("UID:1"))
+    href_a, etag_a = await a.upload(Item("UID:1"))
     status = {}
-    sync(a, b, status, force_delete=True)
+    await sync(a, b, status, force_delete=True)
 
-    ((href_b, etag_b),) = b.list()
-    b.delete(href_b, etag_b)
+    ((href_b, etag_b),) = await aiostream.stream.list(b.list())
+    await b.delete(href_b, etag_b)
     updated = Item("UID:1\nupdated")
-    a.update(href_a, updated, etag_a)
-    sync(a, b, status, force_delete=True)
+    await a.update(href_a, updated, etag_a)
+    await sync(a, b, status, force_delete=True)
 
     assert items(a) == items(b) == {updated.raw}
 
 
-def test_conflict_resolution_invalid_mode():
+@pytest.mark.asyncio
+async def test_conflict_resolution_invalid_mode():
     a = MemoryStorage()
     b = MemoryStorage()
     item_a = Item("UID:1\nitem a")
     item_b = Item("UID:1\nitem b")
-    a.upload(item_a)
-    b.upload(item_b)
+    await a.upload(item_a)
+    await b.upload(item_b)
     with pytest.raises(ValueError):
-        sync(a, b, {}, conflict_resolution="yolo")
+        await sync(a, b, {}, conflict_resolution="yolo")
 
 
-def test_conflict_resolution_new_etags_without_changes():
+@pytest.mark.asyncio
+async def test_conflict_resolution_new_etags_without_changes():
     a = MemoryStorage()
     b = MemoryStorage()
     item = Item("UID:1")
-    href_a, etag_a = a.upload(item)
-    href_b, etag_b = b.upload(item)
+    href_a, etag_a = await a.upload(item)
+    href_b, etag_b = await b.upload(item)
     status = {"1": (href_a, "BOGUS_a", href_b, "BOGUS_b")}
 
-    sync(a, b, status)
+    await sync(a, b, status)
 
     ((ident, (status_a, status_b)),) = status.items()
     assert ident == "1"
@@ -305,7 +322,8 @@ def test_conflict_resolution_new_etags_without_changes():
     assert status_b["etag"] == etag_b
 
 
-def test_uses_get_multi(monkeypatch):
+@pytest.mark.asyncio
+async def test_uses_get_multi(monkeypatch):
     def breakdown(*a, **kw):
         raise AssertionError("Expected use of get_multi")
 
@@ -313,11 +331,11 @@ def test_uses_get_multi(monkeypatch):
 
     old_get = MemoryStorage.get
 
-    def get_multi(self, hrefs):
+    async def get_multi(self, hrefs):
         hrefs = list(hrefs)
         get_multi_calls.append(hrefs)
         for href in hrefs:
-            item, etag = old_get(self, href)
+            item, etag = await old_get(self, href)
             yield href, item, etag
 
     monkeypatch.setattr(MemoryStorage, "get", breakdown)
@@ -326,72 +344,77 @@ def test_uses_get_multi(monkeypatch):
     a = MemoryStorage()
     b = MemoryStorage()
     item = Item("UID:1")
-    expected_href, etag = a.upload(item)
+    expected_href, etag = await a.upload(item)
 
-    sync(a, b, {})
+    await sync(a, b, {})
     assert get_multi_calls == [[expected_href]]
 
 
-def test_empty_storage_dataloss():
+@pytest.mark.asyncio
+async def test_empty_storage_dataloss():
     a = MemoryStorage()
     b = MemoryStorage()
-    a.upload(Item("UID:1"))
-    a.upload(Item("UID:2"))
+    await a.upload(Item("UID:1"))
+    await a.upload(Item("UID:2"))
     status = {}
-    sync(a, b, status)
+    await sync(a, b, status)
     with pytest.raises(StorageEmpty):
-        sync(MemoryStorage(), b, status)
+        await sync(MemoryStorage(), b, status)
 
     with pytest.raises(StorageEmpty):
-        sync(a, MemoryStorage(), status)
+        await sync(a, MemoryStorage(), status)
 
 
-def test_no_uids():
+@pytest.mark.asyncio
+async def test_no_uids():
     a = MemoryStorage()
     b = MemoryStorage()
-    a.upload(Item("ASDF"))
-    b.upload(Item("FOOBAR"))
+    await a.upload(Item("ASDF"))
+    await b.upload(Item("FOOBAR"))
     status = {}
-    sync(a, b, status)
+    await sync(a, b, status)
     assert items(a) == items(b) == {"ASDF", "FOOBAR"}
 
 
-def test_changed_uids():
+@pytest.mark.asyncio
+async def test_changed_uids():
     a = MemoryStorage()
     b = MemoryStorage()
-    href_a, etag_a = a.upload(Item("UID:A-ONE"))
-    href_b, etag_b = b.upload(Item("UID:B-ONE"))
+    href_a, etag_a = await a.upload(Item("UID:A-ONE"))
+    href_b, etag_b = await b.upload(Item("UID:B-ONE"))
     status = {}
-    sync(a, b, status)
+    await sync(a, b, status)
 
-    a.update(href_a, Item("UID:A-TWO"), etag_a)
-    sync(a, b, status)
+    await a.update(href_a, Item("UID:A-TWO"), etag_a)
+    await sync(a, b, status)
 
 
-def test_both_readonly():
+@pytest.mark.asyncio
+async def test_both_readonly():
     a = MemoryStorage(read_only=True)
     b = MemoryStorage(read_only=True)
     assert a.read_only
     assert b.read_only
     status = {}
     with pytest.raises(BothReadOnly):
-        sync(a, b, status)
+        await sync(a, b, status)
 
 
-def test_partial_sync_revert():
+@pytest.mark.asyncio
+async def test_partial_sync_revert():
     a = MemoryStorage(instance_name="a")
     b = MemoryStorage(instance_name="b")
     status = {}
-    a.upload(Item("UID:1"))
-    b.upload(Item("UID:2"))
+    await a.upload(Item("UID:1"))
+    await b.upload(Item("UID:2"))
     b.read_only = True
 
-    sync(a, b, status, partial_sync="revert")
+    await sync(a, b, status, partial_sync="revert")
     assert len(status) == 2
     assert items(a) == {"UID:1", "UID:2"}
     assert items(b) == {"UID:2"}
 
-    sync(a, b, status, partial_sync="revert")
+    await sync(a, b, status, partial_sync="revert")
     assert len(status) == 1
     assert items(a) == {"UID:2"}
     assert items(b) == {"UID:2"}
@@ -399,37 +422,39 @@ def test_partial_sync_revert():
     # Check that updates get reverted
     a.items[next(iter(a.items))] = ("foo", Item("UID:2\nupdated"))
     assert items(a) == {"UID:2\nupdated"}
-    sync(a, b, status, partial_sync="revert")
+    await sync(a, b, status, partial_sync="revert")
     assert len(status) == 1
     assert items(a) == {"UID:2\nupdated"}
-    sync(a, b, status, partial_sync="revert")
+    await sync(a, b, status, partial_sync="revert")
     assert items(a) == {"UID:2"}
 
     # Check that deletions get reverted
     a.items.clear()
-    sync(a, b, status, partial_sync="revert", force_delete=True)
-    sync(a, b, status, partial_sync="revert", force_delete=True)
+    await sync(a, b, status, partial_sync="revert", force_delete=True)
+    await sync(a, b, status, partial_sync="revert", force_delete=True)
     assert items(a) == {"UID:2"}
 
 
 @pytest.mark.parametrize("sync_inbetween", (True, False))
-def test_ident_conflict(sync_inbetween):
+@pytest.mark.asyncio
+async def test_ident_conflict(sync_inbetween):
     a = MemoryStorage()
     b = MemoryStorage()
     status = {}
-    href_a, etag_a = a.upload(Item("UID:aaa"))
-    href_b, etag_b = a.upload(Item("UID:bbb"))
+    href_a, etag_a = await a.upload(Item("UID:aaa"))
+    href_b, etag_b = await a.upload(Item("UID:bbb"))
     if sync_inbetween:
-        sync(a, b, status)
+        await sync(a, b, status)
 
-    a.update(href_a, Item("UID:xxx"), etag_a)
-    a.update(href_b, Item("UID:xxx"), etag_b)
+    await a.update(href_a, Item("UID:xxx"), etag_a)
+    await a.update(href_b, Item("UID:xxx"), etag_b)
 
     with pytest.raises(IdentConflict):
-        sync(a, b, status)
+        await sync(a, b, status)
 
 
-def test_moved_href():
+@pytest.mark.asyncio
+async def test_moved_href():
     """
     Concrete application: ppl_ stores contact aliases in filenames, which means
     item's hrefs get changed. Vdirsyncer doesn't synchronize this data, but
@@ -440,8 +465,8 @@ def test_moved_href():
     a = MemoryStorage()
     b = MemoryStorage()
     status = {}
-    href, etag = a.upload(Item("UID:haha"))
-    sync(a, b, status)
+    href, etag = await a.upload(Item("UID:haha"))
+    await sync(a, b, status)
 
     b.items["lol"] = b.items.pop("haha")
 
@@ -451,7 +476,7 @@ def test_moved_href():
     # No actual sync actions
     a.delete = a.update = a.upload = b.delete = b.update = b.upload = blow_up
 
-    sync(a, b, status)
+    await sync(a, b, status)
     assert len(status) == 1
     assert items(a) == items(b) == {"UID:haha"}
     assert status["haha"][1]["href"] == "lol"
@@ -460,12 +485,13 @@ def test_moved_href():
     # Further sync should be a noop. Not even prefetching should occur.
     b.get_multi = blow_up
 
-    sync(a, b, status)
+    await sync(a, b, status)
     assert old_status == status
     assert items(a) == items(b) == {"UID:haha"}
 
 
-def test_bogus_etag_change():
+@pytest.mark.asyncio
+async def test_bogus_etag_change():
     """Assert that sync algorithm is resilient against etag changes if content
     didn\'t change.
 
@@ -475,27 +501,33 @@ def test_bogus_etag_change():
     a = MemoryStorage()
     b = MemoryStorage()
     status = {}
-    href_a, etag_a = a.upload(Item("UID:ASDASD"))
-    sync(a, b, status)
-    assert len(status) == len(list(a.list())) == len(list(b.list())) == 1
+    href_a, etag_a = await a.upload(Item("UID:ASDASD"))
+    await sync(a, b, status)
+    assert (
+        len(status)
+        == len(await aiostream.stream.list(a.list()))
+        == len(await aiostream.stream.list(b.list()))
+        == 1
+    )
 
-    ((href_b, etag_b),) = b.list()
-    a.update(href_a, Item("UID:ASDASD"), etag_a)
-    b.update(href_b, Item("UID:ASDASD\nACTUALCHANGE:YES"), etag_b)
+    ((href_b, etag_b),) = await aiostream.stream.list(b.list())
+    await a.update(href_a, Item("UID:ASDASD"), etag_a)
+    await b.update(href_b, Item("UID:ASDASD\nACTUALCHANGE:YES"), etag_b)
 
     b.delete = b.update = b.upload = blow_up
 
-    sync(a, b, status)
+    await sync(a, b, status)
     assert len(status) == 1
     assert items(a) == items(b) == {"UID:ASDASD\nACTUALCHANGE:YES"}
 
 
-def test_unicode_hrefs():
+@pytest.mark.asyncio
+async def test_unicode_hrefs():
     a = MemoryStorage()
     b = MemoryStorage()
     status = {}
-    href, etag = a.upload(Item("UID:äää"))
-    sync(a, b, status)
+    href, etag = await a.upload(Item("UID:äää"))
+    await sync(a, b, status)
 
 
 class ActionIntentionallyFailed(Exception):
@@ -511,11 +543,12 @@ class SyncMachine(RuleBasedStateMachine):
     Storage = Bundle("storage")
 
     @rule(target=Storage, flaky_etags=st.booleans(), null_etag_on_upload=st.booleans())
+    @pytest.mark.asyncio
     def newstorage(self, flaky_etags, null_etag_on_upload):
         s = MemoryStorage()
         if flaky_etags:
 
-            def get(href):
+            async def get(href):
                 old_etag, item = s.items[href]
                 etag = _random_string()
                 s.items[href] = etag, item
@@ -526,8 +559,15 @@ class SyncMachine(RuleBasedStateMachine):
         if null_etag_on_upload:
             _old_upload = s.upload
             _old_update = s.update
-            s.upload = lambda item: (_old_upload(item)[0], "NULL")
-            s.update = lambda h, i, e: _old_update(h, i, e) and "NULL"
+
+            async def upload(item):
+                return (await _old_upload(item))[0], "NULL"
+
+            async def update(href, item, etag):
+                return await _old_update(href, item, etag) and "NULL"
+
+            s.upload = upload
+            s.update = update
 
         return s
 
@@ -547,11 +587,11 @@ class SyncMachine(RuleBasedStateMachine):
         _old_upload = s.upload
         _old_update = s.update
 
-        def upload(item):
-            return _old_upload(item)[0], None
+        async def upload(item):
+            return (await _old_upload(item))[0], None
 
-        def update(href, item, etag):
-            _old_update(href, item, etag)
+        async def update(href, item, etag):
+            return await _old_update(href, item, etag)
 
         s.upload = upload
         s.update = update
@@ -590,66 +630,73 @@ class SyncMachine(RuleBasedStateMachine):
         with_error_callback,
         partial_sync,
     ):
-        assume(a is not b)
-        old_items_a = items(a)
-        old_items_b = items(b)
+        async def inner():
+            assume(a is not b)
+            old_items_a = items(a)
+            old_items_b = items(b)
 
-        a.instance_name = "a"
-        b.instance_name = "b"
+            a.instance_name = "a"
+            b.instance_name = "b"
 
-        errors = []
+            errors = []
 
-        if with_error_callback:
-            error_callback = errors.append
-        else:
-            error_callback = None
+            if with_error_callback:
+                error_callback = errors.append
+            else:
+                error_callback = None
 
-        try:
-            # If one storage is read-only, double-sync because changes don't
-            # get reverted immediately.
-            for _ in range(2 if a.read_only or b.read_only else 1):
-                sync(
-                    a,
-                    b,
-                    status,
-                    force_delete=force_delete,
-                    conflict_resolution=conflict_resolution,
-                    error_callback=error_callback,
-                    partial_sync=partial_sync,
+            try:
+                # If one storage is read-only, double-sync because changes don't
+                # get reverted immediately.
+                for _ in range(2 if a.read_only or b.read_only else 1):
+                    await sync(
+                        a,
+                        b,
+                        status,
+                        force_delete=force_delete,
+                        conflict_resolution=conflict_resolution,
+                        error_callback=error_callback,
+                        partial_sync=partial_sync,
+                    )
+
+                for e in errors:
+                    raise e
+            except PartialSync:
+                assert partial_sync == "error"
+            except ActionIntentionallyFailed:
+                pass
+            except BothReadOnly:
+                assert a.read_only and b.read_only
+                assume(False)
+            except StorageEmpty:
+                if force_delete:
+                    raise
+                else:
+                    not_a = not await aiostream.stream.list(a.list())
+                    not_b = not await aiostream.stream.list(b.list())
+                    assert not_a or not_b
+            else:
+                items_a = items(a)
+                items_b = items(b)
+
+                assert items_a == items_b or partial_sync == "ignore"
+                assert items_a == old_items_a or not a.read_only
+                assert items_b == old_items_b or not b.read_only
+
+                assert (
+                    set(a.items) | set(b.items) == set(status)
+                    or partial_sync == "ignore"
                 )
 
-            for e in errors:
-                raise e
-        except PartialSync:
-            assert partial_sync == "error"
-        except ActionIntentionallyFailed:
-            pass
-        except BothReadOnly:
-            assert a.read_only and b.read_only
-            assume(False)
-        except StorageEmpty:
-            if force_delete:
-                raise
-            else:
-                assert not list(a.list()) or not list(b.list())
-        else:
-            items_a = items(a)
-            items_b = items(b)
-
-            assert items_a == items_b or partial_sync == "ignore"
-            assert items_a == old_items_a or not a.read_only
-            assert items_b == old_items_b or not b.read_only
-
-            assert (
-                set(a.items) | set(b.items) == set(status) or partial_sync == "ignore"
-            )
+        asyncio.run(inner())
 
 
 TestSyncMachine = SyncMachine.TestCase
 
 
 @pytest.mark.parametrize("error_callback", [True, False])
-def test_rollback(error_callback):
+@pytest.mark.asyncio
+async def test_rollback(error_callback):
     a = MemoryStorage()
     b = MemoryStorage()
     status = {}
@@ -662,7 +709,7 @@ def test_rollback(error_callback):
     if error_callback:
         errors = []
 
-        sync(
+        await sync(
             a,
             b,
             status=status,
@@ -677,16 +724,22 @@ def test_rollback(error_callback):
         assert status["1"]
     else:
         with pytest.raises(ActionIntentionallyFailed):
-            sync(a, b, status=status, conflict_resolution="a wins")
+            await sync(a, b, status=status, conflict_resolution="a wins")
 
 
-def test_duplicate_hrefs():
+@pytest.mark.asyncio
+async def test_duplicate_hrefs():
     a = MemoryStorage()
     b = MemoryStorage()
-    a.list = lambda: [("a", "a")] * 3
+
+    async def fake_list():
+        for item in [("a", "a")] * 3:
+            yield item
+
+    a.list = fake_list
     a.items["a"] = ("a", Item("UID:a"))
 
     status = {}
-    sync(a, b, status)
+    await sync(a, b, status)
     with pytest.raises(AssertionError):
-        sync(a, b, status)
+        await sync(a, b, status)
